%
% Simple demo for running the
% Multioutput Support Vector Rregression (MSVR)
%
% References:
% "Multioutput support vector regression for
% remote sensing biophysical parameter estimation"
% Tuia, D. and Verrelst, J. and Alonso, L. and Perez-Cruz, F. and Camps-Valls, G.
% IEEE Geoscience and Remote Sensing Letters 8 (4):804-808, 2011
%
%
% Camps-Valls, 2016(c)
% gcamps@uv.es
%

clear;clc;close all;

l = 100;  % number of examples
d = 3;    % input data dimensionality
Q = 2;    % number of output variables to predict

%% Load and prepare data

% Synthetic data
N = 1000;
X = [sin(1:N)' cos(1:N)' randn(N,1)];
Y = [sin(1:N)' cos(1:N)'];

%% Split training-testing data
% Fix seed random generator (important: disable when doing the 100 realizations loop!)
rand('seed',12345);
randn('seed',12345);
rate = 0.1;
[n d] = size(X);                 % samples x dim
r = randperm(n);                 % random index
ntrain = round(rate*n);          % #training samples
Xtrain = X(r(1:ntrain),:);       % training set
Ytrain = Y(r(1:ntrain),:);       % observed training variables
Xtest  = X(r(ntrain+1:end),:);   % test set
Ytest  = Y(r(ntrain+1:end),:);   % observed test variables

%% Remove the mean of Y for training only
Ntrain = size(Xtrain,1);
my      = mean(Ytrain);
Ytrain  = Ytrain - repmat(my,Ntrain,1);

%% Training with pairs Xtrain-Ytrain
% Parameters:
%   C: penalization (regularization) parameter
%   epsilon: tolerated error, defines the insensitivity zone
%   sigma: lengthscale of the RBF kernel
%   tol is a very small number (like 1e-15) for early stopping the
%   optimization of the IRWLS algorithm

ker     = 'rbf';
epsilon = 0.1;
C       = 2;
sigma   = 1;
tol     = 1e-10;
[Beta,NSV,val] = msvr(Xtrain,Ytrain,ker,C,epsilon,sigma,tol);

% Prediction on new test data Xtest
% Build the test kernel matrix
Ktest = kernelmatrix('rbf',Xtest',Xtrain',sigma);
Ntest = size(Xtest,1);
testmeans = repmat(my,Ntest,1);
Ypred = Ktest*Beta + testmeans;

figure,
for q=1:Q
    subplot(Q,1,q),plot(Ytest(:,q),Ypred(:,q),'k.'),title(['Variable ' num2str(q)])
    xlabel('Actual'), ylabel('Prediction')
    res(q)=assessment(Ytest(:,q),Ypred(:,q),'regress');
    res(q).R
end
